// -*- verilog -*-
//
//  USRP - Universal Software Radio Peripheral
//
//  Copyright (C) 2003 Matt Ettus
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


// RAM Interface -- Do we use 512K by 18 or 256K by 36

`define FIFOSIZE 2047 // Remember, FIFO is 2^N-1 words
`define RAMSIZE 131071 // Ditto

module ram_int (input clock, input reset,
		output [18:0] ram_addr, output ram_rw, output ram_we,  // RAM
		inout [31:0] ram_data, inout [3:0] ram_parity,
		output reg [15:0] tx_i_0, output reg [15:0] tx_q_0,  // DUCs
		output reg [15:0] tx_i_1, output reg [15:0] tx_q_1,
		output reg [15:0] tx_i_2, output reg [15:0] tx_q_2,
		output reg [15:0] tx_i_3, output reg [15:0] tx_q_3,
		input [15:0]rx_i_0, input [15:0] rx_q_0,   // DDCs
		input [15:0]rx_i_1, input [15:0] rx_q_1,
		input [15:0]rx_i_2, input [15:0] rx_q_2,
		input [15:0]rx_i_3, input [15:0] rx_q_3,
		input tx_strobe, input rx_strobe,
		input [31:0] tx_fifo_data, output read_ack, input [10:0] txfifolevel, //TXFIFO
		output [31:0] rx_fifo_data, output write_req, input [10:0] rxfifolevel, //RXFIFO
		input [2:0] num_channels,   // FIXME System control
		output [7:0] status // Reports under and overflows,etc.
		);

   // num_channels --> 0 = disabled, 1 = 1, 2 = 2, 3 = all 4
   // TX Radio side goes to DUCs, RX Radio side comes from DDCs
   // TX FIFO side comes from FIFO, RX FIFO side goes to FIFO

   reg txrad_needs_svc, rxrad_needs_svc, txfifo_avail, rxfifo_have_spc;
   reg [17:0] tx_rd_ptr, tx_wr_ptr, rx_rd_ptr, rx_wr_ptr;

   wire [18:0] txram_used = tx_wr_ptr - tx_rd_ptr;  // FIXME handle wrap
   wire [18:0] rxram_used = rx_wr_ptr - rx_rd_ptr;  // FIXME handle wrap
   
   wire txram_empty = (txram_used < num_channels); 
   wire rxram_empty = (rxram_used < num_channels); 
   wire txram_full = (txram_used > `RAMSIZE - num_channels);
   wire rxram_full = (rxram_used > `RAMSIZE - num_channels);

   wire txfifo_burst_ready = (txfifolevel >= num_channels);
   wire rxfifo_ready4burst = (rxfifolevel < `FIFOSIZE - num_channels);

   reg 	need_tx_svc, need_rx_svc;

   wire do_tx_svc = need_tx_svc & ~txram_empty;
   wire do_rx_svc = need_rx_svc & ~rxram_full & ~do_tx_svc;
   wire can_do_fifo = ~do_tx_svc & ~do_rx_svc;
   
   wire do_txfifo_burst = txfifo_burst_ready & ~txram_full & can_do_fifo;
   wire do_rxfifo_burst = rxfifo_ready4burst & ~rxram_empty & can_do_fifo & ~do_txfifo_burst;
   // FIXME need to do proper priority
   
   always @(posedge clock)
     if(reset | tx_strobe)
       need_tx_svc <= #1 1'b1;   // Defaults to needing samples
   //  else if ();
   
   always @(posedge clock)
     if(reset)
       need_rx_svc <= #1 1'b0;
     else if(rx_strobe)
       need_rx_svc <= #1 1'b1;
//     else if() ;
   
   
   always @(posedge clock)
     if(reset)
       begin
	  tx_rd_ptr <= #1 0;
	  tx_wr_ptr <= #1 0;
	  rx_rd_ptr <= #1 0;
	  rx_wr_ptr <= #1 0;
       end
     else
       begin
	  
       end

   
endmodule // ram_int
