// -*- verilog -*-
//
//  USRP - Universal Software Radio Peripheral
//
//  Copyright (C) 2003 Matt Ettus
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

// Two stage interpolator.  
// Assume R = 16 (max), N = 4, M = 1 for each stage for now

// Gain of each stage is R^(N-1), or 16^3 = 4096
// This is 12 bits (maxbitgain)

module cic_interp_2stage(clock,reset,enable,strobe1,strobe2,strobe3,
			 shift1,shift2,signal_in,signal_out);
   parameter bw = 16;
   parameter log2_of_max_rate = 4;
   parameter N = 4;
   parameter maxbitgain = (N-1)*log2_of_max_rate; // FIXME make local param
	
   input clock;
   input reset;
   input enable;
   input strobe1;
   input strobe2;
   input strobe3;
   input [2:0] shift1;
   input [2:0] shift2;
   input [bw-1:0] signal_in;
   output reg [bw-1:0] signal_out;

   wire [bw+maxbitgain-1:0] stage1in,stage2in;
   wire [bw+maxbitgain-1:0] stage1out,stage2out;

   sign_extend #(bw,bw+maxbitgain) 
      ext_input (.in(signal_in),.out(stage1in));

   cic_interp #(.bitwidth(bw+maxbitgain),.stages(N)) 
      stage1(clock,reset,enable,strobe1,strobe2,stage1in,stage1out);

   reg [bw-1:0] s1o_sel;
   always @*
     case(shift1)
       3'd0 : s1o_sel = stage1out[bw-1:0];
       3'd1 : s1o_sel = stage1out[1*(N-1)+bw-1:1*(N-1)];
       3'd2 : s1o_sel = stage1out[2*(N-1)+bw-1:2*(N-1)];
       3'd3 : s1o_sel = stage1out[3*(N-1)+bw-1:3*(N-1)];
       3'd4 : s1o_sel = stage1out[4*(N-1)+bw-1:4*(N-1)];
       default : s1o_sel = stage1out[bw-1:0];
     endcase // case(shift1)
   
   sign_extend #(bw,bw+maxbitgain) 
      ext_s1 (.in(s1o_sel),.out(stage2in));

   cic_interp #(.bitwidth(bw+maxbitgain),.stages(N)) 
      stage2(clock,reset,enable,strobe2,strobe3,stage2in,stage2out);

   always @*
     case(shift2)
       3'd0 : signal_out = stage2out[bw-1:0];
       3'd1 : signal_out = stage2out[1*(N-1)+bw-1:1*(N-1)];
       3'd2 : signal_out = stage2out[2*(N-1)+bw-1:2*(N-1)];
       3'd3 : signal_out = stage2out[3*(N-1)+bw-1:3*(N-1)];
       3'd4 : signal_out = stage2out[4*(N-1)+bw-1:4*(N-1)];
       default : signal_out = stage2out[bw-1:0];
     endcase // case(shift2)
   
endmodule // cic_interp_2stage

