

module halfband_interp 
  (input clock, input reset, input enable,
   input strobe_in, input strobe_out,
   input [15:0] signal_in_i, input [15:0] signal_in_q, 
   output reg [15:0] signal_out_i, output reg [15:0] signal_out_q,
   input coeff_write, input [4:0] coeff_wr_addr, input [15:0] coeff_ram_in);
   
   wire [15:0] 	coeff_ram_in, coeff_ram_out;
   wire [15:0] 	data_ram_out_i;
   wire [15:0] 	data_ram_out_q;
   
   reg [4:0] 	data_rd_addr, data_wr_addr, coeff_rd_addr;

   reg 		filt_done;
   
   wire [15:0] 	mac_out_i;
   wire [15:0] 	mac_out_q;
   
   always @(posedge clock)
     if(reset)
       begin
	  signal_out_i <= #1 16'd0;
	  signal_out_q <= #1 16'd0;
       end
     else if(strobe_in)
       begin
	  signal_out_i <= #1 signal_in_i;   // Multiply by 1 for middle coeff
	  signal_out_q <= #1 signal_in_q;   // Multiply by 1 for middle coeff
       end
     else if(strobe_out)
       begin
	  signal_out_i <= #1 mac_out_i;
	  signal_out_q <= #1 mac_out_q;
       end
   
   always @(posedge clock)
     if(reset)
       coeff_rd_addr <= #1 5'd0;
     else if(coeff_rd_addr != 0)
       coeff_rd_addr <= #1 coeff_rd_addr + 5'd1;
     else if(strobe_in)
       coeff_rd_addr <= #1 5'd1;
   
   always @(posedge clock)
     if(reset)
       data_wr_addr <= #1 5'd0;
     else if(strobe_in)
       data_wr_addr <= #1 data_wr_addr + 5'd1;

   always @(posedge clock)
     if(reset)
       data_rd_addr <= #1 5'd0;
     else if(strobe_in)
       data_rd_addr <= #1 data_wr_addr + 5'd1;
     else if(!filt_done)
       data_rd_addr <= #1 data_rd_addr + 5'd1;

   always @(posedge clock)
     if(reset)
       filt_done <= #1 1'b1;
     else if(strobe_in)
       filt_done <= #1 1'b0;
     else if(data_rd_addr == (data_wr_addr-1))
       filt_done <= #1 1'b1;
   
   ram coeff_ram ( .clock(clock),.write(coeff_write),.wr_addr(coeff_wr_addr),.wr_data(coeff_ram_in),
		   .rd_addr(coeff_rd_addr),.rd_data(coeff_ram_out) );

   ram data_ram_i ( .clock(clock),.write(strobe_in),.wr_addr(data_wr_addr),.wr_data(signal_in_i),
		  .rd_addr(data_rd_addr),.rd_data(data_ram_out_i) );

   ram data_ram_q ( .clock(clock),.write(strobe_in),.wr_addr(data_wr_addr),.wr_data(signal_in_q),
		  .rd_addr(data_rd_addr),.rd_data(data_ram_out_q) );

   mac mac_i (.clock(clock),.reset(reset),.enable(~filt_done),.clear(1'b0),
	    .x(data_ram_out_i),.y(coeff_ram_out),.shift(0),.z(mac_out_i) );

   mac mac_q (.clock(clock),.reset(reset),.enable(~filt_done),.clear(1'b0),
	    .x(data_ram_out_q),.y(coeff_ram_out),.shift(0),.z(mac_out_q) );

endmodule // halfband_interp
