// -*- verilog -*-
//
//  USRP - Universal Software Radio Peripheral
//
//  Copyright (C) 2003 Matt Ettus
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

// Top level module for a full setup with DUCs and DDCs

module dactester
(
 input clk_120mhz,
 input reset,
 input enable_tx,
 input enable_rx,
 input SCLK, 
 input SDI, 
 output SDO, 
 input SEN, 
 input SLD,
 input wire [11:0] adc1_data,
 input wire [11:0] adc2_data,
 input wire [11:0] adc3_data,
 input wire [11:0] adc4_data,
 
 output wire [13:0] dac1_data,
 output wire [13:0] dac2_data,
 output wire [13:0] dac3_data,
 output wire [13:0] dac4_data,
 
 output wire adclk0, 
 output wire adclk1,
 
 output [3:0] adc_oeb,
 input [3:0] adc_otr,
 
 output clk_out,
 
 output [11:0] misc_pins,
 
 // USB interface
 input usbclk,
 input [5:0] usbctl,
 output [5:0] usbrdy,
 inout [15:0] usbdata
 );	
	
	wire daclk = clk_120mhz;
	wire adclk;
	wire [7:0] adc_clock_div, ext_clock_div;
	wire [15:0] bb_tx_i1,bb_tx_q1;
	wire [15:0] bb_rx_i1,bb_rx_q1;
	
	wire [31:0] ch1in_freq, ch1out_freq;
	wire [3:0] interp_rate1, decim_rate1, interp_rate2, decim_rate2;

	clk_divider clk_divider_ext(reset,clk_120mhz,clk_out,ext_clock_div);
	clk_divider clk_divider_adc(reset,clk_120mhz,adclk,adc_clock_div);

	assign adclk0 = adclk;
	assign adclk1 = adclk;
	assign adc_oeb = 4'h0;
	
	wire strobe_interp, strobe_decim;
	
// Debug code
	wire [15:0] i_out;
	wire [15:0] q_out;

	assign dac1_data = {~i_out[15],i_out[14:2]};	
	assign dac2_data = {~q_out[15],q_out[14:2]};	
	assign dac3_data = 14'b0;
	assign dac4_data = 14'b0;

	wire rx_full;
	
	reg enable_tx_reg, enable_rx_reg;
	always @(posedge daclk or posedge reset)
		if(reset) enable_tx_reg <= #1 1'b0;
		else enable_tx_reg <= #1 enable_tx;

	always @(posedge adclk or posedge reset)
		if(reset) enable_rx_reg <= #1 1'b0;
		else enable_rx_reg <= #1 enable_rx;

	bus_interface bus_interface(.usbclk(usbclk),.reset(reset),
		.usbdata(usbdata),.WR(usbctl[0]),.RD(usbctl[1]),.OE(usbctl[2]),
		.txdata({bb_tx_i1,bb_tx_q1}),.rxdata({bb_rx_i1,bb_rx_q1}),
		.txclk(daclk),.rxclk(adclk),
		.txstrobe(strobe_interp),.rxstrobe(strobe_decim),
		.have_space(usbrdy[0]),.have_pkt_rdy(usbrdy[1]),
		.tx_empty(),.rx_full(rx_full),
		.debugbus(misc_pins) );

//FIXME scaling of ADC data?
	ddc ddc_ch1(.clock(adclk),.reset(reset),.enable(enable_rx_reg),.strobe(strobe_decim),
			.freq(ch1in_freq),.i_in({{4{adc1_data[11]}},adc1_data}),.q_in({{4{adc2_data[11]}},adc2_data}),
			.i_out(bb_rx_i1),.q_out(bb_rx_q1) );

	duc duc_ch1(.clock(daclk),.reset(reset),.enable(enable_tx_reg),
			.strobe(strobe_interp),.freq(ch1out_freq),
			.rate1(interp_rate1),.rate2(interp_rate2),
			.i_in(bb_tx_i1),.q_in(bb_tx_q1),
			.i_out(i_out),.q_out(q_out)
			);

	control_bus control_bus(.serial_clock(SCLK),.serial_data_in(SDI),
			.enable(SEN),.load(SLD),.reset(reset),.serial_data_out(SDO),
			.ch1in_freq(ch1in_freq),.ch2in_freq(),.ch3in_freq(),.ch4in_freq(),
			.ch1out_freq(ch1out_freq),.ch2out_freq(),.ch3out_freq(),.ch4out_freq(),
			.adc_clock_div(adc_clock_div),.ext_clock_div(ext_clock_div),
			.interp_rate1(interp_rate1),.interp_rate2(interp_rate2),
			.decim_rate1(decim_rate1),.decim_rate2(decim_rate2),.misc()  );

endmodule
